@@ -23,21 +23,18 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20764 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-08-29 02:15:19 +0000 (Fri, 29 Aug 2008) $
+ * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
  */
 
-#ifndef _PURL_H
-#define _PURL_H
+#ifndef PTLIB_PURL_H
+#define PTLIB_PURL_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
 
 #if P_URL
 
@@ -73,6 +70,9 @@ class PURL : public PObject
       const PFilePath & path   ///< File path to turn into a "file:" URL.
     );
 
+    PURL(const PURL & other);
+    PURL & operator=(const PURL & other);
+
   /**@name Overrides from class PObject */
   //@{
     /**Compare the two URLs and return their relative rank.
@@ -298,10 +298,11 @@ class PURL : public PObject
     );
   //@}
 
-    PBoolean LegacyParse(const PString & _url, const PURLLegacyScheme * schemeInfo);
+    PBoolean LegacyParse(const PString & url, const PURLLegacyScheme * schemeInfo);
     PString LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schemeInfo) const;
 
   protected:
+    void CopyContents(const PURL & other);
     virtual PBoolean InternalParse(
       const char * cstr,         ///< URL as a string to parse.
       const char * defaultScheme ///< Default scheme for URL
@@ -342,8 +343,8 @@ class PURLScheme : public PObject
 class PURLLegacyScheme : public PURLScheme
 {
   public:
-    PURLLegacyScheme(const char * _scheme)
-      : scheme(_scheme) { }
+    PURLLegacyScheme(const char * s)
+      : scheme(s) { }
 
     PBoolean Parse(const PString & url, PURL & purl) const
     { return purl.LegacyParse(url, this); }
@@ -370,6 +371,7 @@ class PURLLegacyScheme : public PURLScheme
 
 #endif // P_URL
 
-#endif // _PURL_H
+#endif // PTLIB_PURL_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
