@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20624 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-07-25 23:56:27 +0000 (Fri, 25 Jul 2008) $
+ * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
  */
 
-#ifndef _ARRAY_H_
-#define _ARRAY_H_
+#ifndef PTLIB_ARRAY_H
+#define PTLIB_ARRAY_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -1250,5 +1250,7 @@ class PBitArray : public PBYTEArray
 };
 
 
-#endif
+#endif // PTLIB_ARRAY_H
+
+
 // End Of File ///////////////////////////////////////////////////////////////
