@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21821 $
+ * $Revision: 21994 $
  * $Author: rjongbloed $
- * $Date: 2008-12-17 06:47:42 +0000 (Wed, 17 Dec 2008) $
+ * $Date: 2009-02-04 16:46:56 -0600 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -281,7 +281,7 @@ PObject::Comparison PAbstractList::Compare(const PObject & obj) const
   PAssert(PIsDescendant(&obj, PAbstractList), PInvalidCast);
   Element * elmt1 = info->head;
   Element * elmt2 = ((const PAbstractList &)obj).info->head;
-  while (elmt1 != NULL && elmt2 != NULL) {
+  while (elmt1 != NULL || elmt2 != NULL) {
     if (elmt1 == NULL)
       return LessThan;
     if (elmt2 == NULL)
