@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21682 $
+ * $Revision: 23236 $
  * $Author: rjongbloed $
- * $Date: 2008-12-02 23:47:17 +0000 (Tue, 02 Dec 2008) $
+ * $Date: 2009-08-20 19:14:22 -0500 (Thu, 20 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -795,6 +795,80 @@ PThread::~PThread()
 }
 
 
+#if PTRACING
+static ULONGLONG ConvertTo64(const FILETIME & ft)
+{
+  ULARGE_INTEGER i;
+  i.HighPart = ft.dwHighDateTime;
+  i.LowPart = ft.dwLowDateTime;
+  return i.QuadPart;
+}
+
+static void OutputTime(ostream & str, ULONGLONG time)
+{
+  ULONGLONG upperbound = 10000;
+  ULONGLONG scale1 = 10;
+  ULONGLONG scale2 = 1;
+  int units = 0;
+  int decimals = 1;
+
+  while (time >= upperbound) {
+    if (--decimals == 0) {
+      if (units == 2) {
+        str << time/scale1 << 's';
+        return;
+      }
+
+      scale1 *= 1000;
+      decimals = 3;
+      ++units;
+    }
+    upperbound *= 10;
+    scale2 *= 10;
+  }
+
+  static const char * const UnitNames[] = { "us", "ms", "s" };
+  str << time/scale1
+      << '.' << setfill('0') << setw(decimals)
+      << (time%scale1)/scale2
+      << UnitNames[units];
+}
+
+static void OutputThreadTimes(const PThread & thread)
+{
+  if (!PTrace::CanTrace(3))
+    return;
+
+  FILETIME created_f, exit_f, kernel_f, user_f;
+  exit_f.dwHighDateTime = exit_f.dwLowDateTime = 0;
+  if (!GetThreadTimes(thread.GetHandle(), &created_f, &exit_f, &kernel_f, &user_f))
+    return;
+
+  ULONGLONG created = ConvertTo64(created_f);
+  ULONGLONG exit    = ConvertTo64(exit_f);
+  ULONGLONG kernel  = ConvertTo64(kernel_f);
+  ULONGLONG user    = ConvertTo64(user_f);
+
+  ostream & output = PTrace::Begin(3, __FILE__, __LINE__);
+  output << "PTLib\tThread ended: name=\"" << thread.GetThreadName() << "\", kernel=";
+  OutputTime(output, kernel);
+  output << ", user=";
+  OutputTime(output, user);
+  output << ", elapsed=";
+  OutputTime(output, exit-created);
+  if (exit > created) {
+    ULONGLONG percent = 1000*(kernel+user)/(exit-created);
+    if (percent > 0)
+      output << " (" << percent/10 << '.' << percent%10 << "%)";
+    else
+      output << " (<0.1%)";
+  }
+  output << PTrace::End;
+}
+#else
+#define OutputThreadTimes(t)
+#endif
+
 void PThread::CleanUp()
 {
   PProcess & process = PProcess::Current();
@@ -805,12 +879,10 @@ void PThread::CleanUp()
   if (!IsTerminated())
     Terminate();
 
-  if (threadHandle != NULL)
+  if (threadHandle != NULL) {
+    OutputThreadTimes(*this);
     CloseHandle(threadHandle);
-
-#if PTRACING
-  PTrace::Cleanup();
-#endif
+  }
 }
 
 
@@ -935,30 +1007,17 @@ void PThread::SetAutoDelete(AutoDeleteFlag deletion)
   autoDelete = deletion == AutoDeleteThread;
 }
 
-#if !defined(_WIN32_WCE) || (_WIN32_WCE < 300)
-#define PTHREAD_PRIORITY_LOWEST THREAD_PRIORITY_LOWEST
-#define PTHREAD_PRIORITY_BELOW_NORMAL THREAD_PRIORITY_BELOW_NORMAL
-#define PTHREAD_PRIORITY_NORMAL THREAD_PRIORITY_NORMAL
-#define PTHREAD_PRIORITY_ABOVE_NORMAL THREAD_PRIORITY_ABOVE_NORMAL
-#define PTHREAD_PRIORITY_HIGHEST THREAD_PRIORITY_HIGHEST
-#else
-#define PTHREAD_PRIORITY_LOWEST 243
-#define PTHREAD_PRIORITY_BELOW_NORMAL 245
-#define PTHREAD_PRIORITY_NORMAL 247
-#define PTHREAD_PRIORITY_ABOVE_NORMAL 249
-#define PTHREAD_PRIORITY_HIGHEST 251
-#endif
 
 void PThread::SetPriority(Priority priorityLevel)
 {
   PAssert(!IsTerminated(), "Operation on terminated thread");
 
   static int const priorities[NumPriorities] = {
-    PTHREAD_PRIORITY_LOWEST,
-    PTHREAD_PRIORITY_BELOW_NORMAL,
-    PTHREAD_PRIORITY_NORMAL,
-    PTHREAD_PRIORITY_ABOVE_NORMAL,
-    PTHREAD_PRIORITY_HIGHEST
+    THREAD_PRIORITY_LOWEST,
+    THREAD_PRIORITY_BELOW_NORMAL,
+    THREAD_PRIORITY_NORMAL,
+    THREAD_PRIORITY_ABOVE_NORMAL,
+    THREAD_PRIORITY_HIGHEST
   };
   SetThreadPriority(threadHandle, priorities[priorityLevel]);
 }
@@ -969,15 +1028,15 @@ PThread::Priority PThread::GetPriority() const
   PAssert(!IsTerminated(), "Operation on terminated thread");
 
   switch (GetThreadPriority(threadHandle)) {
-    case PTHREAD_PRIORITY_LOWEST :
+    case THREAD_PRIORITY_LOWEST :
       return LowestPriority;
-    case PTHREAD_PRIORITY_BELOW_NORMAL :
+    case THREAD_PRIORITY_BELOW_NORMAL :
       return LowPriority;
-    case PTHREAD_PRIORITY_NORMAL :
+    case THREAD_PRIORITY_NORMAL :
       return NormalPriority;
-    case PTHREAD_PRIORITY_ABOVE_NORMAL :
+    case THREAD_PRIORITY_ABOVE_NORMAL :
       return HighPriority;
-    case PTHREAD_PRIORITY_HIGHEST :
+    case THREAD_PRIORITY_HIGHEST :
       return HighestPriority;
   }
   PAssertAlways(POperatingSystemError);
@@ -996,23 +1055,32 @@ void PThread::Yield()
 
 PProcess::HouseKeepingThread::HouseKeepingThread()
   : PThread(1000, NoAutoDeleteThread, NormalPriority, "PTLib Housekeeper")
+  , m_running(true)
 {
   Resume();
 }
 
 
+PProcess::HouseKeepingThread::~HouseKeepingThread()
+{
+  m_running = false;
+  m_breakBlock.Signal();
+  WaitForTermination(500);
+}
+
+
 void PProcess::HouseKeepingThread::Main()
 {
   PProcess & process = PProcess::Current();
 
-  for (;;) {
+  while (m_running) {
 
     // collect a list of thread handles to check, and clean up 
     // handles for threads that disappeared without telling us
     process.deleteThreadMutex.Wait();
     HANDLE handles[MAXIMUM_WAIT_OBJECTS];
     DWORD numHandles = 1;
-    handles[0] = breakBlock.GetHandle();
+    handles[0] = m_breakBlock.GetHandle();
     ThreadList::iterator thread = process.autoDeleteThreads.begin();
     while (thread != process.autoDeleteThreads.end()) {
       if (thread->IsTerminated())
@@ -1078,7 +1146,7 @@ void PProcess::SignalTimerChange()
   if (houseKeeper == NULL)
     houseKeeper = new HouseKeepingThread;
   else
-    houseKeeper->breakBlock.Signal();
+    houseKeeper->m_breakBlock.Signal();
   deleteThreadMutex.Signal();
 }
 
@@ -1162,7 +1230,12 @@ PString PProcess::GetOSName()
           return "Server 2003";
 
         case 6 :
-          return "Vista";
+          switch (info.dwMinorVersion) {
+            case 0 :
+              return "Vista";
+            case 1 :
+              return "7";
+          }
       }
   }
   return "?";
